#!/usr/bin/env python
# -*- coding: utf-8 -*-

# Author:  João S. O. Bueno - jsobueno@gmail.com
# License: LGPL v 3.0
# copyright Joao S. O. Bueno  - 2011

"""
Some toy pure python code to perform matrix operations.

It is 100% useless performance wise, but enough for helping as a matrix calculator,
or some other example code for which it would be nice to have no dependencies.

Running this with pypy instead of cpython will get you ~500% performance improvement.

"""

from __future__ import division
from array import array

class Matrix(object):
    """ matrixes using for transformations - 
        basic matrix arithmetic implemented
    """
    def __init__(self, m=3, n=None, data=None):
        # initialize to identity matrix
        self.m = m
        if n is None:
            n = m
        self.n = n
        
        if data is not None:
            self.data = data
        elif m == n:
            # if we have a square matrix, initate it to the identity matrix
            self.data = [1 if (i % m == i // m) else 0  for i in range(m * n)]
        else:
            self.data = [0 for i in range(m * n)]
            
    def __getitem__(self, (j,i)):
        if i > self.m:
            raise IndexError("Index out of range")
        return self.data[j * self.m + i]
        
    def __setitem__(self, (j,i), value):
        if i > self.m:
            raise IndexError("Index out of range")
        self.data[j * self.m + i] = value
    
    def copy(self):
        return Matrix(self.m, self.n, list(self.data))
        
    def __add__(self, other):
        new = self.copy()
        new += other
        return new
        
    def __iadd__(self, other):
        r = range(self.m)
        for i in range(self.n):
            for j in r:
                self[i,j] += other [i,j]
        return self
        
    def scalar_mul(self, other):
        new = self.copy()
        for i in range(self.m):
            for j in range(self.n):
                new[i,j] *= other
        return new
    
    def __rmul__(self, other):
        return self.scalar_mul(other)
    
    def __mul__(self, other):
        # Verify if multiplication by scalar
        if not hasattr(other, "__getitem__"): 
            return self.scalar_mul(other)
        if self.m != other.n:
            raise ValueError("Incompatible matrix sizes for multiplication")
        new = Matrix(other.m, self.n)
        for self_row in range(self.n):
            for other_col in range(other.m):
                value = 0
                for k in range(self.m):
                    value += self[k, self_row] * other[other_col, k]
                new[other_col, self_row] = value
        return new
        
    def transpose(self):
        new = Matrix(self.n, self.m)
        for i in range(self.m):
            for j in range(self.n):
                new[j,i] = self[i,j]
        return new
    
    def det(self):
        if self.m != self.n:
            raise ValueError("Can't compute determinant of non square matrices")
        if self.m == 2:
            return self[0,0] * self[1,1] - self[0,1] * self[1,0]
        res = 0
        for i in range(self.m):
            p_value = 1
            m_value = 1
            for j in range(self.m):
                p_value *= self[(i + j) % self.m  , j]
                m_index = (self.m - j - 1 - i)
                m_index = m_index if m_index >= 0 else (m_index + self.m)
                m_value *= self [m_index, j]
            res += p_value - m_value
        return res
    
    def cofactor(self, g_i, g_j):
        new = Matrix(self.m - 1, self.n -1)
        ni = -1
        for i in range(self.m):
            if i == g_i: continue
            ni += 1
            nj = -1
            for j in range(self.n):
                if j == g_j: continue
                nj += 1
                new [ni, nj] = self[i, j]
        res =  new.det() * ((-1) ** ((g_i +  g_j)))
        #print g_i, g_j , res
        return res
            
    def cofactor_matrix(self):
        new = Matrix(self.m, self.n)
        for i in range(self.m):
            for j in range(self.n):
                new[i,j] = self.cofactor(i,j)
        return new
    
    def invert(self):
        return self.cofactor_matrix().transpose() * (1 / self.det())
    
    def __repr__(self):
        return "\n".join(" ".join(str(self[col, row]) for col in range(self.m))
             for row in range(self.n)) 
    